
#        Name : CHCANTENARYCURVE

#		 Description:
#			Intended to allow me to create hang fabric between two pinned points for automating cotton growth and to change internal and external spatial configuration

#		 Authors:
#			Carl Holden

#        Usage : Part two of creating hung fabric between two pinned points when automating cotton growth. This involves sheltering the cotton plant from rain leading up to harvest,
#                from overheating and projection from pests during early germination. It also provides entrances, a series of configurable facade to aid in protection against excessive heat
#                gains within a build and create altering spatial variation when walking through and using the space both internally and externally.


###########################################################################################
require 'sketchup.rb'
#------------------------------------------------------------------------------------------


# EXPLANATION:
# Script comprises various methods (defs) wrapped within a 'Module' in order to structure the script.

module CHCANTENARYCURVE

def CHCANTENARYCURVE::cantenary_curve

# EXPLANATION:
# The following block of code triggers the pop-up dialog box which allows users to enter their desired parameters.
# All parameters have been prefixed with '@' to create instance variables in order to allow them to be shared with all methods/definitions. 

def dialog()
    Sketchup::set_status_text("create cantenary line")
    prompts = ["Pinning length ", "Fabric Width ", "fabric curve ", "Type ",]
    @types = ["simple cantenary","offset cantenary"]
   
	
	
# EXPLANATION:
# The following block of code (74-82) tells the default values to remember & re-use the last user-entered values (in session/live).	
	
    @height=50.mm if not @height
    @width=3500.mm if not @width
    @fabric=2800.mm if not @fabric
    @type=@types[0] if not @type
    	
	
    values = [@height, @width, @fabric, @type, ]
    popups = [nil, nil, nil, [@types.join("|")]]
    results = inputbox( prompts, values, popups, "" )
    return nil if not results ### Plugin ends if user Cancels
    @height, @width, @fabric, @type,= results
	
	
# EXPLANATION:
# The following block of code (95-130) divides the sub-dialog pop-ups into individual 'if's' for each @type.
# These additional menus are dependant on answers to '@type' within the first dialog box.

    if @height<=0.0
      UI.messagebox("Height cannot be zero")
      @height=nil
      dialog()
    elsif @width<=0.0
      UI.messagebox("Width cannot be zero")
      @width=nil
      dialog()
    elsif @fabric<=0.0
      UI.messagebox("Fabric curve canot be.n/above height")
      @fabric=nil
      dialog()

    
    end # end of 'if'
    return true
end # end of 'def dialog'

return nil if not dialog()
@section2=true
@type2=false


# EXPLANATION:

# If '@type simple cantinary is selected then def create base will load' selected, plugin loads simple cantenary curve (type 0).
	if @type == @types[0]
	
	elsif @type == @types[1] # if offset cantenary curve is selected then a popup box will appear and it will ask you to confirm 
		prompts = ["Variations "]
        @variations = ["Without deformation", "With deformation"]
        @variation=@variations[0]
        values = [@variation]
        popups = [[@variations.join("|")]]
        results = inputbox prompts, values, popups, "deformation"
        @section2=false if not results
        @variation, = results

    end#if @type

	
# Allows user to cancel dialog and exit.
    if not @section2
      puts "Cantenary Curve Cancelled."
      Sketchup.send_action("selectSelectionTool:")
      return nil
    end #end of 'if'

# All parameters and variables have by now been now entered by the user - arch type 1-21 & dimensions have been specified.


# EXPLANATION:
# The following block of code instantiates the wire-frame geometry and posts loading message on screen.

    @model = Sketchup.active_model
    @model.start_operation "Cantenary Curve"
    entities = @model.active_entities
    @group = entities.add_group
    @entities = @group.entities
    @msg="Calculating Please Wait"
    Sketchup::set_status_text(@msg)


# EXPLANATION:
# The following blocks of code assigns points to draw for the curve based  based on input parameters (@width, @height, @fabric)
# The formulas are intended to approximate a cantenary curve , rather than using circle geometry of .cosh

def create_base
@msg=@msg+". ";Sketchup::set_status_text(@msg)
# Arch Frame (Type 1)
    cs=[]   
    cs[0] = [@width, 0 ,@height]
    cs[1] = [@width, 0, @height]
    cs[2] = [@width, 0, @height] # Arch base
	cs[3] = [@width/(6/5.70), 0, @height + @fabric/4.200] 
	cs[4] = [@width/(6/5.45), 0, @height + @fabric/2.400]
	cs[5] = [@width/(6/5.05), 0, @height + @fabric/1.600]
	cs[6] = [@width/(6/4.40), 0, @height + @fabric/1.240]
	cs[7] = [@width/(6/4.00), 0, @height + @fabric/1.140]
	cs[8] = [@width/(6/3.80), 0, @height + @fabric/1.112]
	cs[9] = [@width/(6/3.50), 0, @height + @fabric/1.084]
	cs[10] = [@width/(6/3.25), 0, @height + @fabric/1.072]
	cs[11] = [@width/(6/3), 0, @height + @fabric/1.066] # Arch peak
	cs[12] = [@width/(6/2.75), 0, @height + @fabric/1.072]
	cs[13] = [@width/(6/2.50), 0, @height + @fabric/1.084]
	cs[14] = [@width/(6/2.20), 0, @height + @fabric/1.112]
	cs[15] = [@width/(6/2.00), 0, @height + @fabric/1.140]
	cs[16] = [@width/(6/1.60), 0, @height + @fabric/1.240]
	cs[17] = [@width/(6/0.95), 0, @height + @fabric/1.600]
	cs[18] = [@width/(6/0.55), 0, @height + @fabric/2.400]
	cs[19] = [@width/(6/0.30), 0, @height + @fabric/4.200] 
    cs[20] = [0, 0, @height] # Arch base
    cs[21] = [0, 0, @height]
	
# adds groups the object as a polyline so that each cantenary curve created at the basepoint does not interact with each other

    @cs=@entities.add_group()
    tents=@cs.entities
    pane=tents.add_curve(cs)

	end # end of 'def create_base' - simple wire-fram cantenary curve
#defines the element	
	def create_offset_cantenary
@msg=@msg+". ";Sketchup::set_status_text(@msg)
# Creates type [offset cantenary when selected from the drop down box]
    co=[]   
    co[0] = [@width,0,@height] 
    co[1] = [@width, 0, @height]
    co[2] = [@width, 0, @height] # curve base
	co[3] = [@width/(6/5.90), 0, @height + @fabric/0.70] 
	co[4] = [@width/(6/5.45), 0, @height + @fabric/0.55]
	co[5] = [@width/(6/5.05), 0, @height + @fabric/0.485]
	co[6] = [@width/(6/4.40), 0, @height + @fabric/0.451]
	co[7] = [@width/(6/4.00), 0, @height + @fabric/0.438]
	co[8] = [@width/(6/3.80), 0, @height + @fabric/0.432]
	co[9] = [@width/(6/3.50), 0, @height + @fabric/0.428]
	co[10] = [@width/(6/3.25), 0, @height + @fabric/0.4265]
	co[11] = [@width/(6/3), 0, @height + @fabric/0.425] # curve peak
	co[12] = [@width/(6/2.75), 0, @height + @fabric/0.4288]
	co[13] = [@width/(6/2.50), 0, @height + @fabric/0.4336]
	co[14] = [@width/(6/2.20), 0, @height + @fabric/0.4450]
	co[15] = [@width/(6/2.00), 0, @height + @fabric/0.4560]
	co[16] = [@width/(6/1.60), 0, @height + @fabric/0.4960]
	co[17] = [@width/(6/0.95), 0, @height + @fabric/0.6400]
	co[18] = [@width/(6/0.55), 0, @height + @fabric/0.9600]
	co[19] = [@width/(6/0.30), 0, @height + @fabric/1.6800] 
    co[20] = [0, 0, @height] # vurve base
    co[21] = [0, 0, @height]


#  groups the object as a polyline so that each cantenary curve created at the basepoint does not interact with each other	
	@co_base=@entities.add_group()
    tents=@co_base.entities
    pane=tents.add_curve(co)
end

# EXPLANATION:
# The following block of code  calls on segments of code to initiate types as defined by the user.

combotype=1
case @type
  when @types[0]
    combotype=1
  when @types[1]
    if @variation=="With Deformation"
      combotype=2
    elsif @variation=="With Deformation"
      combotype=3
    end#if
 else
    puts("Invalid Combo Type ?")
    combotype=1
    return nil
end # end of 'case'


# EXPLANATION:
# The following block of code  on segments of code to enact types in 3D as defined by the user.

case combotype
  when 1
    create_base()
  when 2
    create_offset_cantinary()


end # end of 'case combotype'



# The following block of code makes the group into a component, and defines the component behaviour.


################ unable  to work out how to transform it from the top of the curve instead of using the origin #########################


tr=Geom::Transformation.rotation(ORIGIN,X_AXIS,-180.degrees) # rotates the cantinary curve so that the fabrics lowest point faces down when snapped to a horizontal plane 
@group.transform!(tr)
cantenarycurve2=@model.active_entities.add_group(@group)
@group.explode
@group=cantenarycurve2 #
pt=Geom::Vector3d.new()
tr=Geom::Transformation.translation(pt)
### move origin
@group.entities.transform_entities(tr, @group.entities.to_a)
@msg=@msg+". ";Sketchup::set_status_text(@msg)
###
@msg=@msg+". ";Sketchup::set_status_text(@msg)
@msg=@msg+". ";Sketchup::set_status_text(@msg)
@group.entities.intersect_with(true,@group.transformation,@group,@group.transformation,true,@group)
(@group.entities.to_a).each{|e|
  @msg=@msg+". ";Sketchup::set_status_text(@msg)
  e.explode if e.class==Sketchup::Group
}
@msg=@msg+". ";Sketchup::set_status_text(@msg)
@group.entities.intersect_with(true,@group.transformation,@group,@group.transformation,true,@group)
zfaces=[]
@group.entities.each{|e|
  zfaces<<e if e.class==Sketchup::Face and e.normal.z.abs==1.0
  @msg=@msg+". ";Sketchup::set_status_text(@msg)
}
zfaces.each{|f|
  @msg=@msg+". ";Sketchup::set_status_text(@msg)
}

Sketchup::set_status_text("Please Wait.  Making Gothic Arch Component.")

instance=@group.to_component
component=instance.definition
component.name="Cantinary_curve#1" ### provides a name that will increment #2 etc
component.description="Cantenary conponent: " ### this allows me to update the conponents name and add aditional information for example the height and width of it.

### set its behavior so it glues etc - snaps to planes
behavior=component.behavior
behavior.cuts_opening=false #stops it from cutting an opening if placed against a wall
behavior.is2d=true #i2d= method is used to set the 2D behavior for a component: whether it can be glued or aligned to a face.
behavior.snapto=SnapTo_Horizontal ### = snaps to a floor or support beam only  this is the horizontal plane:alternatives= _Arbitrary [any], _vertical, _ Sloped
component.insertion_point=Geom::Point3d.new(0, 0, @height)
instance.erase! if instance.valid? #allows me to keep adding more conponents 
Sketchup::set_status_text("")
### add component to the cursor for easy addition to a wall
@model.place_component(component, true)
#-----------------------------------------------------------------------
     @model.commit_operation


# EXPLANATION:
# The following block of code (1260-1277) makes the group into a component, and defines the component behaviour.

end # end of 'def cantenary_curve'



	end# end of module


#-----------------------------------------------------------------------------
if not file_loaded?(File.basename(__FILE__))
    UI.menu("Plugins").add_item("cantenary curve"){CHCANTENARYCURVE::cantenary_curve}
end
#-----------------------------------------------------------------------------
file_loaded(File.basename(__FILE__))
